# Install the latest release :candy:

If you're on Mac OS X, upgrade to the latest version with:

```sh
$ brew install scalacenter/bloop/bloop
```

Otherwise, run:

```
$ curl -L https://github.com/scalacenter/bloop/releases/download/v1.0.0-M11/install.py | python
```

Read the complete instructions in our [Installation page][installation instructions].

# Highlights :books:

## [Redesign the configuration format][#529] (by [Jorge Vicente Cantero][@jvican])

With Bloop 1.0 being around the corner, we need to fix all the issues with the current configuration
file format, and come up with a solution that will be stable for a long time.

Jorge has been working towards a definitive design for what the configuration file should look like,
and his intensive head scratching resulted in [this pull request][#529].

## Mill integration (by [Olivier Mélois][@Baccata]) :factory:

Bloop now includes a [Mill integration][#532], which means that your build definition can be exported from
Mill's format to Bloop's.

To try the Mill integration, add the following to the `build.sc` of a project:

```scala
import $ivy.`ch.epfl.scala::mill-bloop:1.0.0-M11`
```

And then run `mill bloop.integrations.mill.Bloop/install` from the directory of the mill project.

## Support for Scala.js and Scala Native :robot:

This release of Bloop includes support for both [Scala.js][#526] and [Scala Native][#457],
contributed by [Tim Nieradzik][@tindzk], [Jorge Vicente Cantero][@jvican] and [Martin
Duhem][@Duhemm].

Using Scala Native or Scala.js with Bloop is completely transparent. If your project uses any of
these platforms, Bloop will automatically configure itself to link and run using the appropriate
tools and runtime.

At the moment, running the test is not supported and waiting for [a new release of Scala
Native][native-pr-1234].

## [Export sbt build information][#538] (by [Jorge Vicente Cantero][@jvican])

When sbt is launched with the sbt-bloop plugin, it will now immediately export Bloop configuration
files for the build and meta build. This allows a BSP server to directly pick up the changes and
notify its client that a build has changed.

This enhancement is crucial for BSP and integration with clients, and improves that overall user
experience.

## [Export sources and docs][#527] (by [Jorge Vicente Cantero][@jvican])

The Bloop configuration file now contains paths to the documentation and sources of the dependencies
of a build. This allows a BSP client to query them from Bloop, and lets IDEs offer a better user
experience.

## Features :building_construction:

#### [Print test reports][#499] (by [Tim Nieradzik][@tindzk])

Bloop now prints a full report of test execution after the command `test` finishes. This gives you
information about the tests that ran, the failures, errors and successes.

Also, Bloop now correctly reports the outcome of your tests in its exit code.

## Bugfixes :bug:

#### [Kebabize flag names in autocompletion][#487] (by [Martin Duhem][@Duhemm])

All the flags that Bloop accepts on its command line interface should be written using kebab-case.
Unfortunately, a bug in our autocompletions made Bloop provide suggestions that were using camelCase
instead.

#### [Support relative config directories in `--config-dir`][#486] (by [Marko Svaljek][@msvaljek])

Bloop supports an option called `--config-dir`, which defines where Bloop should look for your
configuration files (the default value is `.bloop` in your current working directory).

When this option was passed, Bloop was creating an absolute path from the base directory where the
Bloop server had been started, instead of doing so from your current working directory. This patch
fixes this incorrect behavior!

## Usability improvements

#### [Improved error reporter][#537] (by [Gabriele Petronella][@gabro])

Bloop features a nice error reporter that tries to unclutter your screen and make the most useful
bits pop out of your screen. Unfortunately, our reporter was unfriendly to some terminals, and broke
links from error messages to source. Thankfully, [@gabro][@gabro] fixed that!

## Integration improvements:

### [sbt](https://github.com/sbt/sbt/)

#### [Add option to configure where to store analysis file][#502] (by [Marko Svaljek][@msvaljek])

The location where the state of the incremental compiler should be stored can now be configured.

#### [Export custom configurations][#512] (by [Paweł Bartkiewicz][@tues])

Our plugin for sbt used to export only the `Compile` and `Test` configurations. [@tues][@tues]
improved it so that builds can configure which configuration should be exported, and have Bloop
configuration files generated.

### [Maven](https://maven.apache.org)

#### Support Java-only projects [#51][#511] and [#524][#524] (by [Tobias Roeser][@lefou] and [Jorge Vicente Cantero][@jvican])

Our Maven integration used to have issues with Java-only projects. Thanks to this fix by
[@lefou][@lefou], those are now correctly supported by Bloop!

## Documentation improvements :books:

#### [Document how to debug Bloop][#530] (by [Paweł Bartkiewicz][@tues])

Our developer documentation was still missing information on how to debug Bloop. [@tues][@tues] has
been contributing the missing bits!

#### [Advanced sbt configuration][#523] (by [Paweł Bartkiewicz][@tues])

With the recent improvements to our sbt integrations, our documentation still needed to catch up.
The documentation has been updated to reflect the latest additions and capabilities of the sbt
integration, and how to configure it properly.

#### Fix typos in the documentation

Errors easily get into documentation... Thankfully, our community is here to help. Thank you
all for fixing our mistakes:

 - [#503][#503] by [Alexander Samsig][@asamsig]
 - [#517][#517] by [Stephen Nancekivell][@stephennancekivell]
 - [#513][#513] by [Gabriele Petronella][@gabro]
 - [#535][#535] by [@travisltq][@travisltq]

#### [Add documentation for Maven integration][#489] (by [Paweł Bartkiewicz][@tues])

Even though our Maven integration has been around for a long time, we still hadn't updated our
documentation to tell our users how to use it. The documentation is now up-to-date!

#### [Mention `--coursier` in developer documentation][#488] (by [Paweł Bartkiewicz][@tues])

The installation script allows to specify the version of coursier that should be used during
installation. However, the documentation wasn't mentioning it.

## Meta-improvements :hot_pepper:

#### [Upgrade nuProcess][#525] (by [Jorge Vicente Cantero][@jvican])

[nuProcess][nuprocess] is the library that we use to spawn processes. Jorge worked on migrating to
the latest version of nuProcess, which includes fixes for bugs that affected Bloop.

#### [Speed up compilation time by 8x][#509] (by [Jorge Vicente Cantero][@jvican])

Using [scalac-profiling][scalac-profiling], Jorge has been able to determine how to dramatically
improve compilation time of Bloop itself. Thanks to his modifications, Bloop can now be compiled
*much* faster!

#### [Fix compilation warnings][#479] (by [Jorge Vicente Cantero][@jvican])

There were some warnings in our codebase. No more! Enjoy a quiet compilation.

## Contributors :busts_in_silhouette:

According to `git shortlog -sn --no-merges v1.0.0-M10..1.0.0-M11`, 11 people
contributed to this release: Jorge Vicente Cantero, Martin Duhem, Paweł Bartkiewicz, Tim Nieradzik,
Marko Svaljek, Gabriele Petronella, Olivier Mélois, travisltq, Stephen Nancekivell, Tobias Roeser,
Alexander Samsig

Thank you all :sparkles:!

[installation instructions]: https://scalacenter.github.io/bloop/docs/installation
[configuration]: https://scalacenter.github.io/bloop/docs/configuration-format/

[#540]: https://github.com/scalacenter/bloop/pull/540
[#538]: https://github.com/scalacenter/bloop/pull/538
[#537]: https://github.com/scalacenter/bloop/pull/537
[#532]: https://github.com/scalacenter/bloop/pull/532
[#535]: https://github.com/scalacenter/bloop/pull/535
[#530]: https://github.com/scalacenter/bloop/pull/530
[#529]: https://github.com/scalacenter/bloop/pull/529
[#528]: https://github.com/scalacenter/bloop/pull/528
[#527]: https://github.com/scalacenter/bloop/pull/527
[#526]: https://github.com/scalacenter/bloop/pull/526
[#457]: https://github.com/scalacenter/bloop/pull/457
[#525]: https://github.com/scalacenter/bloop/pull/525
[#524]: https://github.com/scalacenter/bloop/pull/524
[#523]: https://github.com/scalacenter/bloop/pull/523
[#521]: https://github.com/scalacenter/bloop/pull/521
[#513]: https://github.com/scalacenter/bloop/pull/513
[#517]: https://github.com/scalacenter/bloop/pull/517
[#512]: https://github.com/scalacenter/bloop/pull/512
[#511]: https://github.com/scalacenter/bloop/pull/511
[#479]: https://github.com/scalacenter/bloop/pull/479
[#499]: https://github.com/scalacenter/bloop/pull/499
[#509]: https://github.com/scalacenter/bloop/pull/509
[#502]: https://github.com/scalacenter/bloop/pull/502
[#503]: https://github.com/scalacenter/bloop/pull/503
[#495]: https://github.com/scalacenter/bloop/pull/495
[#494]: https://github.com/scalacenter/bloop/pull/494
[#489]: https://github.com/scalacenter/bloop/pull/489
[#488]: https://github.com/scalacenter/bloop/pull/488
[#487]: https://github.com/scalacenter/bloop/pull/487
[#486]: https://github.com/scalacenter/bloop/pull/486

[scala-native-pr-1234]: https://github.com/scala-native/scala-native/pull/1234
[nuprocess]: https://github.com/brettwooldridge/NuProcess

[@asamsig]: https://github.com/asamsig
[@Baccata]: https://github.com/Baccata
[@Duhemm]: https://github.com/Duhemm
[@gabro]: https://github.com/gabro
[@jvican]: https://github.com/jvican
[@lefou]: https://github.com/lefou
[@msvaljek]: https://github.com/msvaljek
[@tindzk]: https://github.com/tindzk
[@stephennancekivell]: https://github.com/stephennancekivell
[@travisltq]: https://github.com/travisltq
[@tues]: https://github.com/tues
